#if __GNUC__ && __AVR__
#include <Arduino.h>
#include <Ultrasonic.h>

//// Adafruit Display
#include <SPI.h>
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>

#define OLED_RESET 4
Adafruit_SSD1306 display(OLED_RESET);
#define NUMFLAKES 10
#define XPOS 0
#define YPOS 1
#define DELTAY 2

#if (SSD1306_LCDHEIGHT != 64)
#error("Height incorrect, please fix Adafruit_SSD1306.h!");
#endif

#endif

#include "Nibble_arr.h"
#include <string.h>

#ifndef __AVR__
#include <math.h>
#include <cstdio>
#include <iostream>
#include <unistd.h>
#endif

const int PIN_ENA = 0;
const int PIN_ENB = 1;
const int PIN_IN1 = 3; 
const int PIN_IN2 = 5;
const int PIN_IN3 = 9;
const int PIN_IN4 = 10; 

const int SERVO_PIN = 7; 

Nibble_arr test;



int distance = 21;


int getX(int b, int alpha) {
  return cos((M_PI/180)*alpha) * b;
}

int getY(int b, int alpha) {
  return sin((M_PI/180)*alpha) * b;
}


#if __GNUC__ && __AVR__
int motorPower = 255;
Ultrasonic ultrasonic(A2,A3);

  
// put your setup code here, to run once:



void setup() {
  Serial.begin(9600);
  pinMode(PIN_ENA,OUTPUT); 
  pinMode(PIN_ENB, OUTPUT);
  pinMode(PIN_IN1, OUTPUT);
  pinMode(PIN_IN2, OUTPUT);
  pinMode(PIN_IN3, OUTPUT);
  pinMode(PIN_IN4, OUTPUT);
  pinMode(SERVO_PIN, OUTPUT);
  digitalWrite(PIN_ENA, HIGH);
  digitalWrite(PIN_ENB, HIGH);

  // Display Initialisieren
  display.begin(SSD1306_SWITCHCAPVCC, 0x3C);
  display.clearDisplay();
  display.setTextSize(2);
  display.setTextColor(WHITE);
  display.setCursor(0,0);
  display.println("2WD China Bots");
  //display.setTextSize(3);
  display.println("HTWK-");
  display.setTextSize(2);
  display.println("Leipzig");
  display.display();
  Serial.println("Hello.");
}

void dbg_println(char* val){
  Serial.println(val);
}

/**
 * @param leftMotor -255 bis 255, Stärke
 * @param rightMotor -255 bis 255, Stärke
 */
void setMotors(int leftMotor, int rightMotor) {
  if (rightMotor>=0)
  {
    analogWrite(PIN_IN2, rightMotor);
    digitalWrite(PIN_IN1, LOW);
  } else {
    analogWrite(PIN_IN2, rightMotor);
    digitalWrite(PIN_IN1, HIGH);
  }

  if (leftMotor>=0)
  {
    analogWrite(PIN_IN4, leftMotor);
    digitalWrite(PIN_IN3, LOW);
  } else {
    analogWrite(PIN_IN4, leftMotor);
    digitalWrite(PIN_IN3, HIGH);
  }

}

void stopMotors() {
  digitalWrite(PIN_IN1,LOW);
  digitalWrite(PIN_IN2,LOW);
  digitalWrite(PIN_IN3,LOW);
  digitalWrite(PIN_IN4,LOW);
  digitalWrite(PIN_ENA,LOW);
  digitalWrite(PIN_ENB,LOW);
}

void enableHigh() {
  digitalWrite(PIN_ENA,HIGH);
  digitalWrite(PIN_ENB,HIGH);
}



void setServo(int degrees) {
  int lenMicroSecondsOfPulse = (float) degrees/ 90.0 * 1000 + 300;
  int lenMicroSecondsOfPeriod = 20 * 1000; // 20 milliseconds (ms) 
  for (int i = 0; i < 200; ++i) {
    digitalWrite(SERVO_PIN, HIGH); 
    // Delay for the length of the pulse
    delayMicroseconds(lenMicroSecondsOfPulse);

    // Turn the voltage low for the remainder of the pulse
    digitalWrite(SERVO_PIN, LOW);

    // Delay this loop for the remainder of the period so we don't
    // send the next signal too soon or too late
    delayMicroseconds(lenMicroSecondsOfPeriod - lenMicroSecondsOfPulse); 

  }

}

int scanDirection(int angle) {
  setServo(angle);
  delay(200);
  return ultrasonic.distanceRead();
}

void setDisplaytext(String text, Adafruit_SSD1306 display) {
  display.clearDisplay();
  display.setTextSize(2);
  display.setTextColor(WHITE);
  display.setCursor(0,0);
  display.println(text);
  display.display();
}

int lookLeft(){
  setServo(180);
  delay(1000);
  return ultrasonic.distanceRead();
}

int lookRight(){
  setServo(0);
  delay(1000);
  return ultrasonic.distanceRead();
}

#endif

#ifndef __AVR__
  int testData[] = {10,20,20,20,10,20,60,80,100,110,10,32,42,18,15,16,17,0};
  int scanDirection(int i) {
    //return testData[i/10];
    return 20;
  }

  void dbg_println(char* val){
    std::cout << val << std::endl;
  }

  void setDisplaytext(char* text) {
    std::cout<<"[LCD]: "<<text;
  }

#endif

void loop() {
  char s[50];
  for (int i = 0; i <= 180; i+=10)
  {
    int distance = scanDirection(i);
    int x = getX(distance, i);
    int y = getY(distance, i);
    test.increment((x/SCALE)+SIZE/2,y/SCALE);
    sprintf(s,"X: %d, Y: %d, b: %d, alpha: %d\n",x,y,distance,i);
    dbg_println(s);
    //setDisplaytext(s);
    dbg_println("============ARRAY================");
    char* tmpstring;
    for (unsigned char i = SIZE; i > 0; --i)
    {
      for (unsigned char j = 0; j < SIZE; ++j)
      {
        #if __GNUC__ && __AVR__
          Serial.print(+test.get(j,i));
          Serial.print(" ");
        #else
          std::cout << +test.get(j, i) << " ";
        #endif
      }
      #if __GNUC__ && __AVR__
        Serial.println();
      #else
        std::cout << std::endl;
      #endif
    }
    dbg_println("============ARRAY=ENDE===========");
    {
      
    };
    #ifndef __AVR__
      usleep(500000);
    #endif
  }
}


#ifndef __AVR__

int main()
{ 
 //setup(); //Not needed (at least until now)
 while(1){loop();}
}


#endif




